% Author: Dong Chu
% Email: chudong@whu.edu.cn
% The codes are created based on the method described in the following paper
% Chu, D., Shen, H., Guan, X., Chen, J.M., Li, X., Li, J., Zhang, L., 2021.
% Long time-series NDVI reconstruction in cloud-prone regions via spatio-temporal tensor completion. Remote Sens. Environ. 264, 112632.
% https://doi.org/https://doi.org/10.1016/j.rse.2021.112632

dir_modis='Y:\NDVI_TC\SEAsia\h27v07\'; %% input folder that includes all hdf files of MOD13A2 from 2001 to 2018
ny=18; %% number of years, which is 18 in this study
nd=23; %% number of observations in a year, which is 23 for MOD13A2


dir_input=dir(fullfile(dir_modis,'*.hdf'));
file_Names={dir_input.name};
fullfile_Names=fullfile(dir_modis,file_Names);
image_num=length(fullfile_Names);
exam_info=hdfinfo(fullfile_Names{1});
exam_ndvi=hdfread(exam_info.Vgroup.Vgroup(1).SDS(1));
exam_flag=hdfread(exam_info.Vgroup.Vgroup(1).SDS(12));
[col,row]=size(exam_ndvi);
org_modis=zeros(col,row,image_num,'int16');
org_flag=zeros(col,row,image_num,'int8');

for b=1:image_num
    file_info=hdfinfo(fullfile_Names{b});
    org_modis(:,:,b)=hdfread(file_info.Vgroup.Vgroup(1).SDS(1));
    org_flag(:,:,b)=hdfread(file_info.Vgroup.Vgroup(1).SDS(12));
end

myPool = parpool(4);
%% ST-Tensor
res_modis_ST=Tensor_Completion(org_modis,org_flag,ny,nd); %% Spatio-temporal tensor completion
res_modis_ST=iterative_l1tf(res_modis_ST,org_flag); %% iterative l1 trend filtering
write_ENVI(single(res_modis_ST),'h27v07_ST-Tensor_result')